<'
package shr_spi3;

//Configuration unit for the eVC.  All configuration fields should be constrained 
//in this unit, and are fed "downward" to the rest of the eVC.
unit shr_spi3_agent_config_u like any_unit {

   numports : uint;  	      	       //Number of ports for this agent.
   keep numports           <= SHR_SPI3_MAX_PORTS;     // Number of ports is limited by configuration define

   start_port : uint;	      	       //Port on which to send data if no
   keep soft start_port == 0; 	       //xSX signal is transmitted before data

   direction : shr_spi3_direction_t;   //Direction of the agent (TX/RX)

   valid_cycles_after_ENB : list of uint;     //Number of cycles AFTER enb
   keep valid_cycles_after_ENB.size() == num_ports_supported;
   keep for each in valid_cycles_after_ENB {
      soft it in [1..3];
   };
   
   enable_dtpa : list of shr_spi3_bool_s; //Enable the use of the DTPA signals for handshaking
   keep enable_dtpa.size() == numports;
   keep for each in enable_dtpa {
      it.port_id == active_ports[index];
   };
   
   enable_stpa : list of shr_spi3_bool_s; //Enable the use of the STPA signal for handshaking
   keep enable_stpa.size() == numports;
   keep for each in enable_stpa {
      soft it.data != enable_dtpa[index].data;
      it.port_id == active_ports[index];
   };
   
   enable_ptpa : list of shr_spi3_bool_s; //Enable the use of PTPA signals for handshaking
   keep enable_ptpa.size() == numports;
   keep for each in enable_ptpa {
      soft it.data == enable_stpa[index].data;
      it.port_id == active_ports[index];
   };
   
   ignore_tpa : list of shr_spi3_bool_s; //Ingore all xTPA signals, send traffic regardless of their state
   keep ignore_tpa.size() == numports;
   keep for each in ignore_tpa {
      soft it.data == FALSE;
      it.port_id == active_ports[index];
   };
   
   parity : shr_spi3_parity_mode_t;    //Parity mode for this agent.
   
   num_ports_supported : uint;	       //Number of ports supported by the DUT
   keep num_ports_supported <= SHR_SPI3_MAX_PORTS;

   //List of the ative port numbers for tihs agent
   active_ports : list of uint;        
   keep active_ports.size() == numports;
   
   all_ports : list of uint;
   keep all_ports.size() == num_ports_supported;
   keep for each in all_ports {
      it == index;
   };
   
   mixed_ports : list of uint;
   keep mixed_ports.is_a_permutation(all_ports);
   
   keep soft active_ports == mixed_ports[0..numports-1];
   
   mode : shr_spi3_bus_mode_t;	    //Bus mode - NARROW, WIDE
   
   xSX_mode : shr_spi3_xSX_cycle_kind_t;  //xSX cycle mode -   FIXED, RANDOM, DISABLED
   keep soft xSX_mode not in [DISABLED];
   
   xSX_after_every_packet : bool;            	//Change ports after every packet.
   
   interleave_length : uint;  	       //If xSX_mode is FIXED, this is the number of transfer after which an xSX will be issued
   keep soft interleave_length in [32..1500];
   
   arbitration : shr_spi3_arb_method_t;   //Arbitration mode for this agent.
   keep soft arbitration   == RR;                     // Default is to use basic round-robin arbitration

   arb_weights : list of uint;                        // Arbitration weights used in weighted round-robin arbitration

   keep for each (weight) in arb_weights {
        arbitration == RR  => weight == 1;             // Force all arbitration weights to be equal for basic round-robin
        arbitration == WRR => soft weight in [1..16];  // Set soft limits on weights for weighted round-robin arbitration
   };
   keep gen (arbitration) before (arb_weights);       // Insure implied keep constraints aren't interpreted "backwards"

   keep arb_weights.size() == numports;               // Insure there is precisely one arbitration weight for each port
       
};

extend RX shr_spi3_agent_config_u {

   keep for each in ignore_tpa {
      it.data == TRUE;
   };

};

'>
