<'
package shr_spi3;

//The drive_spi3_protocol() method gets a SPI3 transfer, which represents one clock cycle
//of SPI3 activity and drives it to the DUT via ports that are instantiated in
//the agent.  This method is different for each LAYER/DIRECTION pair (i.e. there 
//are four different implementations) based on what signals of the SPI3 interface
//need to be driven for that particular layer/direction.


extend LINK'layer TX'direction shr_spi3_bfm_u {


    drive_spi3_protocol() @clock is also {
        var next_cycle : shr_spi3_datacycle_s = new;

        wait @reset_end;
        while TRUE {
	    if agent is a TX LINK shr_spi3_agent_u (agent) then {
               next_cycle           = cycle_driver.get_next_item();
               if cycle_driver.bypass_sequence_driver == FALSE then {
	          emit cycle_driver.item_done;	       
	       };
               agent.sig_tsx$       = next_cycle.xfer_start;
               agent.sig_tenb$      = next_cycle.enable;
               agent.sig_tsop$      = next_cycle.sop;
               agent.sig_teop$      = next_cycle.eop;
               agent.sig_terr$      = next_cycle.error;
               agent.sig_tprty$     = next_cycle.parity;
               agent.sig_tmod$      = next_cycle.modulus;
               agent.sig_tadr$      = next_cycle.address;
	       
	       if config.mode == NARROW then {
                  agent.sig_tdat$ = %{24'b0,next_cycle.data[7:0]};
	       } else if config.mode == WIDE then {
                  agent.sig_tdat$ = next_cycle.data;
               };
	    };
	 if next_cycle.xfer_start == 1 and next_cycle.enable == 1 then {
	    transfers_since_xSX = 1;
	    gen next_random_xSX;
	 } else if next_cycle.enable == 0 then {
	    transfers_since_xSX = transfers_since_xSX + 1;
	 };
         wait cycle;
        }; //while TRUE
    };//drive_spi3_protocol
};

extend PHY'layer RX'direction shr_spi3_bfm_u {
   drive_spi3_protocol() @clock is also {
      var next_cycle : shr_spi3_datacycle_s = new;

      wait @reset_end;
      while TRUE {
         
	 if agent is a RX PHY shr_spi3_agent_u (agent) then {
      	    if (last_renb != 1'b1) {
      	       next_cycle         = cycle_driver.get_next_item();
      	       emit cycle_driver.item_done;
               agent.sig_rsx$       = next_cycle.xfer_start;
               agent.sig_rval$      = next_cycle.valid;
               agent.sig_rsop$      = next_cycle.sop;
               agent.sig_reop$      = next_cycle.eop;
               agent.sig_rerr$      = next_cycle.error;
               agent.sig_rprty$     = next_cycle.parity;
               agent.sig_rmod$      = next_cycle.modulus;
               if config.mode == NARROW then {
                  agent.sig_rdat$ = %{24'b0,next_cycle.data[7:0]};
	       } else if config.mode == WIDE then {
                  agent.sig_rdat$ = next_cycle.data;
               };
	    };
	    if next_cycle.xfer_start == 1 and last_renb == 1 then {
	       transfers_since_xSX = 1;
	       gen next_random_xSX;
   	    } else if next_cycle.enable == 0 then {
	       transfers_since_xSX = transfers_since_xSX + 1;
	    };

            last_renb = agent.sig_renb$;
	 };//if agent....
         wait cycle;	 
      };//while TRUE
   };//drive_spi3_protocol
}; //extend PHY RX shr_spi3_bfm_u

extend LINK RX shr_spi3_bfm_u {
   drive_spi3_protocol() @clock is also {
      var next_cycle : shr_spi3_datacycle_s = new;

      wait @reset_end;
      while TRUE {
         
	 if agent is a RX LINK shr_spi3_agent_u (agent) then {
      	    next_cycle         = cycle_driver.get_next_item();
	    emit cycle_driver.item_done;
	    agent.sig_renb$ = next_cycle.enable;
	 };
	 wait [next_cycle.valid_clocks]*cycle;
      };//while TRUE
   };//drive_spi3_protocol
}; //extend LINK RX shr_spi3_bfm_u

extend PHY TX shr_spi3_bfm_u {
   drive_spi3_protocol()@clock is also {
      var next_cycle : shr_spi3_datacycle_s = new;
      
      wait @reset_end;
      while TRUE {
      	 if agent is a TX PHY shr_spi3_agent_u (agent) then {
	    next_cycle = cycle_driver.get_next_item();
	    emit cycle_driver.item_done;
	    agent.sig_dtpa$ = next_cycle.dtpa;
	    agent.sig_stpa$ = next_cycle.stpa;
	    agent.sig_ptpa$ = next_cycle.ptpa;
	 };
	 wait [next_cycle.valid_clocks]*cycle;
      };
   };//drive_spi3_protocol
};//extend PHY TX shr_spi3_bfm_u



'>

