<'
package shr_spi3;

sequence shr_spi3_bfm_data_sequence using
   item = shr_spi3_datacycle_s;

extend LINK TX shr_spi3_bfm_u {
   cycle_driver : shr_spi3_bfm_data_sequence_driver is instance;
   on clock {
      emit cycle_driver.clock;
   };
};

extend PHY RX shr_spi3_bfm_u {
   cycle_driver : shr_spi3_bfm_data_sequence_driver is instance;
   on clock {
      emit cycle_driver.clock;
   };
};

extend LINK RX shr_spi3_bfm_u {
   cycle_driver : shr_spi3_bfm_data_sequence_driver is instance;
   keep soft cycle_driver.sequence.kind == DEFAULT_RX_LINK;
   on clock {
      emit cycle_driver.clock;
   };
}; 

extend PHY TX shr_spi3_bfm_u {
   cycle_driver : shr_spi3_bfm_data_sequence_driver is instance;
   on clock {
      emit cycle_driver.clock;
   };
};  

extend shr_spi3_bfm_data_sequence_driver {
   layer : shr_spi3_layer_t;
   keep layer == agent.layer;
      
   direction : shr_spi3_direction_t;
   keep direction == agent.direction;

   agent : ACTIVE shr_spi3_agent_u;
   keep agent == get_enclosing_unit(shr_spi3_agent_u);      

   bfm : shr_spi3_bfm_u;
   keep bfm == get_enclosing_unit(shr_spi3_bfm_u);
   
   eop_flag : bool;  	      	             //last transfer was an EOP
   keep eop_flag == TRUE;
   
   lose_next_sop : bool;
   keep lose_next_sop == FALSE;
   !lose_next_sop_port : uint;
   
   lose_next_eop : bool;
   keep lose_next_eop == FALSE;
   !lose_next_eop_port : uint;
   
   corrupt_next_xSX : bool;
   keep corrupt_next_xSX == FALSE;
   
   switch_ports() : bool is {
      var switch : bool;
      //Begin by determining if the current port is still valid:
           
      	    //Check to see if the port is valid based on interleave length
      	    if agent.config.xSX_mode == FIXED then {
      	       if bfm.transfers_since_xSX >= agent.config.interleave_length then {
	          switch = TRUE;
	       };
      	    }
      	    else if agent.config.xSX_mode == RANDOM then {
      	       if bfm.transfers_since_xSX >= bfm.next_random_xSX then {
	          switch = TRUE;
	       };
      	    };
      
      	    //If the agent is configured to change ports after each packet
      	    //Check for the EOP flag (set in the BFM sequence);
      	    if agent.config.xSX_after_every_packet == TRUE and eop_flag == TRUE then {
      	       switch = TRUE;
      	    };

	    //Start-up condition - is the bfm.active_port even in the config.active_ports list?
      	    if agent.data_ports.has(it.port_id == bfm.active_port) then {
	 
	       //Check the port status
	       //Only bother with xTPA if not ignoring xTPA
	       if agent.config.ignore_tpa.first(it.port_id == bfm.active_port).data == FALSE then {
	          //If both STPA and DTPA are enabled, STPA takes precedence
	          if agent.config.enable_stpa.first(it.port_id == bfm.active_port).data == TRUE then {
	             //Check the last captured value of STPA for this port
	             if agent.port_status.first(it.port_id == bfm.active_port).stpa_status == 0 then {
	             	switch = TRUE;
	             };
	          } else if agent.config.enable_dtpa.first(it.port_id == bfm.active_port).data == TRUE then {
	             //Check the last captured value of DTPA for this port
	             if agent.port_status.first(it.port_id == bfm.active_port).dtpa_status == 0 then {
	             	switch = TRUE;
	             };
	          };//else if....
      	       };//if agent.config.ignore_tpa
      	    }
      	    else {
      	       switch = TRUE;
      	    };

      	    //Check to see if the port is valid based on data available from the port driver
      	    if switch == FALSE then {
      	       //If none of the ports have data, then there is no need to switch to any port
      	       if agent.data_ports.first(it.port_id == bfm.active_port).data_avail == FALSE then {
      	          switch = TRUE;
      	       }
      	       else {
      	          switch = FALSE;
      	       };
      	    };
      result = switch;
   }; //switch_ports()
   
   get_new_port() : shr_spi3_arb_result_s @clock is {
      var port_found : bool;
      port_found = FALSE;

      var ports_tried : list of bool;
      gen ports_tried keeping {
      	 it.size() == agent.config.numports;
      };
      
      var arb_result : shr_spi3_arb_result_s;
      arb_result = new;
      
      for each in ports_tried {
      	 ports_tried[index] = FALSE;
      };
      
      //Determine if any of the ports have data available
      if agent.data_ports.data_avail.has(it==TRUE) then {      
            
      	 while (ports_tried.has(it == FALSE) and (port_found == FALSE)) {
      	    if (me is a RX PHY shr_spi3_bfm_data_sequence_driver (driver)) then {
      	       all of {
      	          {arb_result = deep_copy(driver.arb.driver.get_next_item());};
      	          {emit driver.arb.driver.clock;};
      	       };
      	       emit driver.arb.driver.item_done;
	       
      	    }
      	    else if (me is a TX LINK shr_spi3_bfm_data_sequence_driver (driver)) then {
      	       all of {
      	          {arb_result = deep_copy(driver.arb.driver.get_next_item());};
      	          {emit driver.arb.driver.clock;};
      	       };
      	       emit driver.arb.driver.item_done;
      	    };
      	 
	    //Check to see if the chosen port has data available
	    if agent.data_ports.first(it.port_id == arb_result.port_id).data_avail == TRUE then {
      	       //Check to see if the chosen port is ignoring xTPA or not.
	       if agent.config.ignore_tpa.first(it.port_id == arb_result.port_id).data == TRUE then {
	          port_found = TRUE;
		  bfm.active_port = arb_result.port_id;
		  eop_flag = FALSE; 
	       }
	       else {
		  //Check whether the port is in DTPA or PTA mode for packet start
	          //And also check if the applicable xTPA signal is high
      	          if (((agent.config.enable_dtpa.first(it.port_id == arb_result.port_id).data == TRUE) and
	             (agent.port_status.first(it.port_id == arb_result.port_id).dtpa_status == 1)) or
		     ((agent.config.enable_ptpa.first(it.port_id == arb_result.port_id).data == TRUE) and
		     ( agent.port_status.first(it.port_id == arb_result.port_id).ptpa_status == 1))) then {
      	             	port_found = TRUE;
      	             	bfm.active_port = arb_result.port_id;
			eop_flag = FALSE;
	          };
      	       };
	    };
	    //Add the INDEX of the agent.config.active_ports list for the attempted
	    //port to the ports tried list.  This way only that all of the INDEXES
	    //have been tried needs to be tracked.
      	    ports_tried[agent.config.active_ports.first_index(it==arb_result.port_id)] = TRUE;
      	 };//while
      } else {//none of the ports have data available
      	 port_found = FALSE;
      };
      
      arb_result.port_found = port_found;
      result = deep_copy(arb_result);
   };//get new port

};

extend shr_spi3_bfm_data_sequence  {
   layer : shr_spi3_layer_t;
   direction : shr_spi3_direction_t;
   keep layer == driver.layer;
   keep direction == driver.direction;
};

extend RX PHY shr_spi3_bfm_data_sequence_driver {   
   arb : shr_spi3_arbiter_u;
   keep agent is a ACTIVE PHY RX shr_spi3_agent_u (agent) => arb == agent.arb;
};

extend TX LINK shr_spi3_bfm_data_sequence_driver {
   arb : shr_spi3_arbiter_u;
   keep agent is a ACTIVE LINK TX shr_spi3_agent_u (agent) => arb == agent.arb ;
};

extend RANDOM shr_spi3_datacycle_s {


};//RANDOM shr_spi3_datacycle_s

extend LAYERED shr_spi3_datacycle_s {

};//LAYERED shr_spi3_datacycle_s

extend shr_spi3_bfm_data_sequence_kind : [RANDOM_PAYLOAD, LAYERED_PAYLOAD] ;

extend RANDOM_PAYLOAD shr_spi3_bfm_data_sequence {
   !datacycle : RANDOM shr_spi3_datacycle_s ;
   body() @driver.clock is only {
      do datacycle ;
   };
};


extend MAIN shr_spi3_bfm_data_sequence {
   keep sequence.kind != SIMPLE ;
};

extend LAYERED_PAYLOAD shr_spi3_bfm_data_sequence {

   !datacycle : OUT LAYERED shr_spi3_datacycle_s;
   
   has_parity_error : bool;
   keep soft has_parity_error == FALSE;
   
   has_xERR : bool;
   keep soft has_xERR == FALSE;
   
   body() @driver.clock is only {
      
      message(HIGH, "SPI3 eVC LAYERED_PAYLOAD shr_spi3_bfm_data_sequence body started.");
   
      var curr_data : shr_spi3_inport_bfmdata_s;
      var arb_result : shr_spi3_arb_result_s;
      var switch_ports : bool;

      switch_ports = driver.switch_ports();
      
      message(HIGH, "driver.switch_ports() Called and Returned");
      message(HIGH, switch_ports);
            
      if switch_ports == TRUE then {
      	 //It has been decided that the current port is no longer valid.
	 //A new port must be found.
	 //All ports should be tried before an idle cycle is driven to the bus
	 
	 arb_result=deep_copy(driver.get_new_port());
	 
	 
	 //After exiting this loop, either a valid port was found, or no valid port was found
	 if ((arb_result.port_found == FALSE) or 
	    ((arb_result.port_found == TRUE) and (driver.agent.config.xSX_mode == DISABLED)))then {
	    message(MEDIUM, "DOing an idle cycle");
      	    do datacycle keeping {
      	       it.xfer_start == 0;
	       it.enable == 1;
	       it.valid == 0;
      	    };
	 }
	 else if (arb_result.port_found == TRUE and driver.agent.config.xSX_mode != DISABLED) then {
	    //send out the port switch transfer
	    message(MEDIUM, "DOing a xSX cycle");
	    
	    if (has_parity_error == TRUE) or
	       (driver.corrupt_next_xSX == TRUE) then {
	          driver.corrupt_next_xSX = FALSE;
		  has_parity_error = TRUE;
	     } else {
	       has_parity_error = FALSE;
	    };
	    
	    do datacycle keeping {
	       it.xfer_start == 1;
	       it.enable == 1;
	       it.valid == 0;
	       it.data == driver.bfm.active_port;
	       it.error == 0;
	       it.bad_parity == has_parity_error;
	    };
	 };
      }
      else if switch_ports == FALSE {
	    if driver.agent is a ACTIVE shr_spi3_agent_u (agent) then {
	    //we're still on the same port, so just drive the data
	    curr_data = driver.agent.data_ports.first(it.port_id == driver.bfm.active_port).get_bfm_data();
	    if curr_data.eop == 1 then {
	       driver.eop_flag = TRUE;
	    };
	    
	    if driver.lose_next_sop == TRUE and 
	       driver.bfm.active_port == driver.lose_next_sop_port and
	       curr_data.sop == 1 then {
	          curr_data.sop = 0;
		  driver.lose_next_sop = FALSE;
      	    };
	    
	    if driver.lose_next_eop == TRUE and 
	       driver.bfm.active_port == driver.lose_next_eop_port and
	       curr_data.eop == 1 then {
	          curr_data.eop = 0;
		  driver.lose_next_sop = FALSE;
      	    };
		  
            message(MEDIUM, "DOing a data cycle");
	    if has_xERR == TRUE and curr_data.eop == 1 then {
	       do datacycle keeping {
	          it.xfer_start == 0;
	          it.enable == 0;
	          it.valid == 1;
	          it.sop == curr_data.sop;
	          it.eop == curr_data.eop;
	          it.error == has_xERR.as_a(bit);
	          it.data == curr_data.data;
	          it.modulus == curr_data.mod;
	          it.bad_parity == has_parity_error;
	       };//do keeping
	    }//if has_xERR == TRUE
	    else {
	       do datacycle keeping {
	          it.xfer_start == 0;
	          it.enable == 0;
	          it.valid == 1;
	          it.sop == curr_data.sop;
	          it.eop == curr_data.eop;
	          it.error == curr_data.error.as_a(bit);
	          it.data == curr_data.data;
	          it.modulus == curr_data.mod;
	          it.bad_parity == has_parity_error;
	       };//do keeping
      	    };//else	       
	 };
      };
   };//body@driver.clock

}; // LAYERED_PAYLOAD shr_spi3_datacyle_s

extend shr_spi3_bfm_data_sequence_kind : [DEFAULT_RX_LINK];

extend DEFAULT_RX_LINK shr_spi3_bfm_data_sequence {
   !datacycle : shr_spi3_datacycle_s ;
   body() @driver.clock is only {
      do datacycle keeping {
      	 it.enable == 0;
	 it.valid_clocks in [5..20];
      };
   };  
};

extend shr_spi3_bfm_data_sequence_driver {

    calculate_parity(data : uint(bits:32)) : bit is {
        case agent.config.parity {
            ODD  : {result = bitwise_xnor(data);};
            EVEN : {result = bitwise_xor(data);};
        };
    };
};

//By default, calculate the correct parity for each transaction.
//Since this is a soft constraint, it can be easily overwritten.
extend OUT'port_direction shr_spi3_datacycle_s {
       post_generate() is also {
         if bad_parity == TRUE then {
	    parity = ~driver.calculate_parity(data);
	 } else {
	    parity = driver.calculate_parity(data);
	 };
      };
};

extend shr_spi3_bfm_data_sequence_driver {
   bypass_sequence_driver : bool;
   keep soft bypass_sequence_driver == FALSE;
   
   when TRUE'bypass_sequence_driver TX'direction LINK'layer shr_spi3_bfm_data_sequence_driver {
      post_generate() is also {
      	 gen_and_start_main = FALSE;
      };
      
      get_next_item(): shr_spi3_datacycle_s @clock is only {
      	 var curr_data : shr_spi3_inport_bfmdata_s;
      	 var arb_result : shr_spi3_arb_result_s;
      	 var switch_ports : bool;

      	 switch_ports = switch_ports();
      
      	 message(HIGH, "driver.switch_ports() Called and Returned");
      	 message(HIGH, switch_ports);
            
      	 if switch_ports == TRUE then {
      	    //It has been decied that the current port is no longer valid.
	    //A new port must be found.
	    //All ports should be tried before an idle cycle is driven to the bus
	 
	    arb_result=deep_copy(get_new_port());
	 
	 
	    //After exiting this loop, either a valid port was found, or no valid port was found
	    if arb_result.port_found == FALSE then {
	       message(MEDIUM, "DOing an idle cycle");
      	       gen result keeping {
	          it.driver == me;
	       };
      	       result.xfer_start = 0;
	       result.enable = 1;
	       result.valid = 0;
	    }
	    else if arb_result.port_found == TRUE then {
	       //send out the port switch transfer
	       message(MEDIUM, "DOing a xSX cycle");
	       gen result keeping {
	          it.driver == me;
	       };
	       result.xfer_start = 1;
	       result.enable = 1;
	       result.valid = 0;
	       result.data = bfm.active_port;
	       result.error = 0;
	    };
      	 }
      	 else if switch_ports == FALSE {
	    if agent is a ACTIVE shr_spi3_agent_u (agent) then {
	       //we're still on the same port, so just drive the data
	       curr_data = agent.data_ports.first(it.port_id == bfm.active_port).get_bfm_data();
	       if curr_data.eop == 1 then {
	          eop_flag = TRUE;
	       };
               message(MEDIUM, "DOing a data cycle");
	       gen result keeping {
	          it.driver == me;
	       };
	       result.xfer_start = 0;
	       result.enable = 0;
	       result.valid = 1;
	       result.sop = curr_data.sop;
	       result.eop = curr_data.eop;
	       result.error = curr_data.error.as_a(bit);
	       result.data = curr_data.data;
	       result.modulus = curr_data.mod;
	       result.parity = calculate_parity(result.data);
	    };
	 };
      };//gen next item
   };//when
};//extend

'>
